/* $Id: id_attrs.c,v 1.4 2023/03/28 05:08:34 leavens Exp $ */
// Attributes of identifiers in the symbol table
#include <stdlib.h>
#include <stddef.h>
#include "utilities.h"
#include "id_attrs.h"

// Return a freshly allocated id_attrs struct
// with the given file location and the rest of the fields uninitialized.
// If there is no space, bail with an error message,
// so this should never return NULL.
id_attrs *id_attrs_start(file_location floc)
{
    id_attrs *ret = (id_attrs *)malloc(sizeof(id_attrs));
    if (ret == NULL) {
	bail_with_error("No space to allocate id_attrs!");
    }
    ret->file_loc = floc;
    return ret;
}

// Return a freshly allocated id_attrs struct
// with its field tok set to t, var_type set to vt,
// and its offset to ofst.
// If there is no space, bail with an error message,
// so this should never return NULL.
id_attrs *id_attrs_loc_create(file_location floc, var_type vt,
			      unsigned int ofst)
{
    id_attrs *ret = id_attrs_start(floc);
    ret->file_loc = floc;
    ret->vt = vt;
    ret->loc_offset = ofst;
    return ret;
}
