/* $Id: instruction.h,v 1.2 2023/03/30 02:14:01 leavens Exp $ */
#ifndef _INSTRUCTION_H
#define _INSTRUCTION_H
#include <stdio.h>
#include <stdbool.h>
#include "machine_types.h"

typedef struct {
    int op; /* opcode */
    word m; /* M */
} instruction;

extern bool stop_reading;
extern instruction read_instruction(FILE *);

// Is the argument a legal op code for the machine?
extern bool legal_op_code(int op);

// Requires: legal_op_code(op);
// Return the ASCII mnemonic for the op code given.
extern const char * mnemonic(int op);

// Requires: out is open and writable
// print the header of the output table to out
extern void instruction_heading_print(FILE *out);

// Requires: out is an open FILE
// print the instruction with mnemonic (on one line)
// followed by a newline character.
void instruction_debug_format_print(FILE *out, instruction instr);

// Requires: out is an open FILE
// print the address on out followed by the instruction (on one line)
// followed by a newline character.
void instruction_debug_print_with_addr(FILE *out, int addr, instruction instr);

// Requires: out is an open FILE
// print the instruction in the VM's input format,
// followed by a newline character.
void instruction_vm_format_print(FILE *out, instruction instr);

#endif
