/* $Id: unparser.h,v 1.5 2023/03/29 20:46:56 leavens Exp $ */
#ifndef _UNPARSER_H
#define _UNPARSER_H
#include <stdio.h>
#include "ast.h"

// Unparse the given program AST with output going to the file out
extern void unparseProgram(FILE *out, AST *ast);

// Unparse the AST_list of var-decls to out
// with the given nesting level
// (note that if ast_list_is_empty(vds), then nothing is printed)
extern void unparseVarDecls(FILE *out, AST_list vds, int level);

// Unparse a single var-decl given by the AST vd to out,
// indented for the given nesting level
extern void unparseVarDecl(FILE *out, AST *vd, int level);

// Unparse the statement given by the AST stmt to out,
// indented for the given level.
extern void unparseStmt(FILE *out, AST *stmt, int indentLevel);

// Unparse the assignment statment given by stmt to out
// with indentation level given by level,
extern void unparseAssignStmt(FILE *out, AST *stmt, int level);

// Unparse the sequential statment given by stmt to out
// with indentation level given by level (indenting the body one more level)
extern void unparseBeginStmt(FILE *out, AST *stmt, int level);

// Unparse the list of statments given by stmt to out
// with indentation level given by level.
extern void unparseStmtList(FILE *out, AST *stmt1, AST_list rest,
			    int level);

// Unparse the if-statment given by stmt to out
// with indentation level given by level (and each body indented one more)
extern void unparseIfStmt(FILE *out, AST *stmt, int level);

// Unparse the read statment given by stmt to out
extern void unparseReadStmt(FILE *out, AST *stmt, int level);

// Unparse the write statment given by stmt to out.
extern void unparseWriteStmt(FILE *out, AST *stmt, int level);

// Unparse the expression given by the AST exp to out
// adding parentheses to indicate the nesting relationships
extern void unparseExpr(FILE *out, AST *exp);

// Unparse the expression given by the AST exp to out
// adding parentheses (whether needed or not)
extern void unparseBinExpr(FILE *out, AST *exp);

// Unparse the given identifer reference (use) to out
extern void unparseIdent(FILE *out, AST *id);

// Unparse the given number to out in decimal format
extern void unparseNumber(FILE *out, AST *num);

// Unparse the given logical negation expression to out
extern void unparseNotExpr(FILE *out, AST *exp);

#endif
