/* $Id: machine_types.c,v 1.2 2023/03/29 01:27:19 leavens Exp $ */
#include <stdio.h>
#include <math.h>
#include "machine_types.h"
#include "utilities.h"

// Requires: out is open for writing
// Print to out the word w using format "%-5d" or "%-5f"
// as appropriate for the type tag
void word_print(FILE *out, word w)
{
    if (w.type_tag == int_type) {
	fprintf(out, "%-5d", w.data.i);
    } else {
	// assert(w.type_tag == float_type);
	fprintf(out, "%-5f", w.data.f);
    }
}

// Return the int as a word
word word_from_int(int i)
{
    word w;
    w.type_tag = int_type;
    w.data.i = i;
    return w;
}

// Return the float as a word
word word_from_float(float f)
{
    word w;
    w.type_tag = float_type;
    w.data.f = f;
    return w;
}


// Return the word as an integer, checking the type tag
int word_as_int(word w)
{
    if (w.type_tag != int_type) {
	bail_with_error("word_as_int passed non-int!");
    }
    return w.data.i;
}

// Return the word as an integer,
// rounding if it was a float
int word2int(word w)
{
    switch (w.type_tag) {
    case int_type:
	return word_as_int(w);
	break;
    case float_type:
	return round(word_as_float(w));
	break;
    default:
	bail_with_error("word2int doesn't understand the type tag!");
	// the following should never execute
	return 0;
    }
}

// Return the word as a float, checking the type tag
float word_as_float(word w)
{
    if (w.type_tag != float_type) {
	bail_with_error("word_as_float passed non-float!");
    }
    return w.data.f;
}

// Return the word as a float,
// extending an int to a float if need be
float word2float(word w)
{
    switch (w.type_tag) {
    case float_type:
	return word_as_float(w);
	break;
    case int_type:
	return word_as_int(w); // C will promote to a float
	break;
    default:
	bail_with_error("word2float doesn't understand the type tag!");
	// the following should never execute
	return 0.0;
    }

}
